
function [p25] = port5by5SESortVW(P1, btm, P, Ret)

% port5by5SESortVW: Construct 25 value-weighted portfolios sorting on P1 and btm
% 
% INPUT:
%   P1: N by T matrix of sorting variables
%   P : N by T matrix of firm capitalization
%   btm: N by T matrix of sorting variables
%   Ret: N by T matrix of individual firm return
%   Rf: risk free rate

% OUTPUT:
%   p25: 
% 
%  Xiaoji lin and Lu Zhang 2012

% warning off

% Initializing
[N, T]  = size(Ret);
pointer = round([1; N*.20; N*.40; N*.60; N*.80; N]);  % pointers used in sorting

[N T]   = size(Ret);
T       = floor(T/12)*12;
P       = P(:,1:T);        
P1      = P1(:,1:T);        
btm     = btm(:,1:T);     

% further initializing
lag   = 0;
p25   = zeros([5 5 T - lag]);           % 25 portfolio returns formed on p1 and btm

% construct 25 portfolio returns
for t = lag + 1 : T
    % sorting and rebalancing once every year
   if (mod(t - 1, 12) == 0)       % using monthly returns
        % pick up size and btm recorded at the beginning of a month
         svec    = P1(:, t);
         btmvec  = btm(:, t);
         IndStru = ff93Sorting(svec, btmvec, pointer);
        % construct value-weighted ff25 portfolio returns 
        tSpan            = t : t + 11;         % using monthly returns
        [p25(:, :, tSpan - lag)] = p25Constr(IndStru, tSpan, P, Ret);
   end 
     
end

%------------------------------subfunction declaration-----------------------------
% 
function IndStru = ff93Sorting(svec, mvec, pointer)

% First sort on size then within each size portfolios sort on
% book-to-market vector find the indices (of stocks in the panel) for the
% 25 constructed portfolios

% OUTPUT:
%   IndStru : structural cell (matrix of matrices of difference lengths)
% 
%  Xiaoji lin and Lu Zhang 2012

% Initializing
IndStru  = struct('idp11', [], 'idp12', [], 'idp13', [], 'idp14', [], 'idp15', [], ...
                  'idp21', [], 'idp22', [], 'idp23', [], 'idp24', [], 'idp25', [], ...
                  'idp31', [], 'idp32', [], 'idp33', [], 'idp34', [], 'idp35', [], ...
                  'idp41', [], 'idp42', [], 'idp43', [], 'idp44', [], 'idp45', [], ...
                  'idp51', [], 'idp52', [], 'idp53', [], 'idp54', [], 'idp55', []     );
              
% size sort
[ssize, sInd] = sort(svec);     clear ssize
ids1    = sInd(pointer(1)     : pointer(2));
ids2    = sInd(pointer(2) + 1 : pointer(3));
ids3    = sInd(pointer(3) + 1 : pointer(4));
ids4    = sInd(pointer(4) + 1 : pointer(5));
ids5    = sInd(pointer(5) + 1 : pointer(6));     clear sInd

% construct structural cells for second variable in each 3 ids1,...ids3
  mvecnew1 = mvec(ids1); [sm, mInd] = sort(mvecnew1);  mInd1 = ids1(mInd);  clear sm mInd
  mvecnew2 = mvec(ids2); [sm, mInd] = sort(mvecnew2);  mInd2 = ids2(mInd);  clear sm mInd
  mvecnew3 = mvec(ids3); [sm, mInd] = sort(mvecnew3);  mInd3 = ids3(mInd);  clear sm mInd
  mvecnew4 = mvec(ids4); [sm, mInd] = sort(mvecnew4);  mInd4 = ids4(mInd);  clear sm mInd
  mvecnew5 = mvec(ids5); [sm, mInd] = sort(mvecnew5);  mInd5 = ids5(mInd);  clear sm mInd 

[N, T]  = size(mvecnew1); pointer1 = round([0; N*.20; N*.40; N*.60; N*.80; N]);  clear N    
[N, T]  = size(mvecnew2); pointer2 = round([0; N*.20; N*.40; N*.60; N*.80; N]);  clear N
[N, T]  = size(mvecnew3); pointer3 = round([0; N*.20; N*.40; N*.60; N*.80; N]);  clear N
[N, T]  = size(mvecnew4); pointer4 = round([0; N*.20; N*.40; N*.60; N*.80; N]);  clear N
[N, T]  = size(mvecnew5); pointer5 = round([0; N*.20; N*.40; N*.60; N*.80; N]);  clear N

% construct structural cells for 25 p1 and btm : IndStru.idIkrate11,...IndStru.idIkrate25
for i = 1 : 5
    for j = 1 : 5
       
            if (i==1); eval(['IndStru.idp' num2str(i) num2str(j) '=mInd' num2str(i) '(pointer1(j)+1 : pointer1(j+1)'  ');']); end
            if (i==2); eval(['IndStru.idp' num2str(i) num2str(j) '=mInd' num2str(i) '(pointer2(j)+1 : pointer2(j+1)'  ');']); end
            if (i==3); eval(['IndStru.idp' num2str(i) num2str(j) '=mInd' num2str(i) '(pointer3(j)+1 : pointer3(j+1)'  ');']); end   
            if (i==4); eval(['IndStru.idp' num2str(i) num2str(j) '=mInd' num2str(i) '(pointer4(j)+1 : pointer4(j+1)'  ');']); end
            if (i==5); eval(['IndStru.idp' num2str(i) num2str(j) '=mInd' num2str(i) '(pointer5(j)+1 : pointer5(j+1)'  ');']); end   
    end
end
  
clear pointer

%---------------------------------------------------------------------------
function [p25]   = p25Constr(IndStru, tSpan, P, Ret);

%  p25Constr:
%   construct 25 portfolios using structural cell index across the time range in tSpan
%
% OUTPUT:
%   p25 : 5 by 5 by length(tSpan) portfolio returns
% 
%  Xiaoji lin and Lu Zhang 2012


[N, T]  = size(Ret);
p25 = zeros([5 5 length(tSpan)]);
for i = 1 : 5
    for j = 1 : 5
        % value-weighted returns
         eval(['p25(' num2str(i) ',' num2str(j) ', :) = sum(P(IndStru.idp' num2str(i) num2str(j) ...
            ', tSpan).*Ret(IndStru.idp' num2str(i) num2str(j) ', tSpan)) ./sum(P(IndStru.idp' ...
            num2str(i) num2str(j) ', tSpan));']);
       
        % equal-weighted returns
        % eval(['p25(' num2str(i) ',' num2str(j) ', :) = mean(Ret(IndStru.idp' num2str(i) num2str(j) ', tSpan));']);
    end
end