function [p6] = portSortVW6(P, btm, Ret)

% PORTSORTVW6: Construct Fama and French 6 size/bm value-weighted portfolios
% 
% INPUT:
%   P: N by T matrix of firm capitalization
%   btm: N by T matrix of book-to-market of firms
%   Ret: N by T matrix of individual firm return

% OUTPUT:
%   p6: 
% 
%  Xiaoji lin and Lu Zhang 2012
% warning off

% Initializing
[N, T]  = size(Ret);
pointer1 = round([1; N*.50; N]);        % pointers used in sorting
pointer2 = round([1; N*.3; N*.7; N]);   % pointers used in sorting
clear N T

[N T]   = size(Ret);
T       = floor(T/12)*12;
RetNew  = Ret(:,1:T);
P       = P(:,1:T);      % weight: june at year t
btm     = btm(:,1:T);    % bm: dec at year t-1


% further initializing
lag   = 0;
p6   = zeros([2 3 T - lag]);    % 6 portfolio returns formed on size and book-to-market

% construct 25 portfolio returns
for t = lag + 1 : T
    % sorting and rebalancing once every year
     if (mod(t - 1, 12) == 0)       % using monthly returns
        % pick up size and btm recorded at the beginning of a month
         svec    = P(:, t);
         btmvec  = btm(:, t);
         IndStru = ff93Sorting(svec, btmvec, pointer1,pointer2);
        % construct value-weighted ff6 portfolio returns 
        tSpan            = t : t + 11;         % using monthly returns
        [p6(:, :, tSpan - lag)] = p6Constr(IndStru, tSpan, P, RetNew);
     end 
end


%------------------------------subfunction declaration-----------------------------
% 
function IndStru = ff93Sorting(svec, btmvec, pointer1,pointer2)

% FF93SORTING:
%   given size and book-to-market vector find the indices (of stocks in the panel) for the
% 6 constructed portfolios
%
% OUTPUT:
%   IndStru : structural cell (matrix of matrices of difference lengths)
% 
%  Xiaoji lin and Lu Zhang 2011

% Initializing
IndStru  = struct('idp11', [], 'idp12', [], 'idp13', [], 'idp21', [], 'idp22', [], 'idp23', [] );
% size sort
[ssize, sInd] = sort(svec);     clear ssize
ids1    = sInd(pointer1(1)     : pointer1(2));
ids2    = sInd(pointer1(2) + 1 : pointer1(3)); clear sInd
% book-to-market sort
[sbtm, bInd] = sort(btmvec);    clear sbtm
idb1    = bInd(pointer2(1)     : pointer2(2));
idb2    = bInd(pointer2(2) + 1 : pointer2(3));
idb3    = bInd(pointer2(3) + 1 : pointer2(4)); clear bInd
% take index intersections to form 25 portfolios
for i = 1 : 2
    for j = 1 : 3
       eval(['IndStru.idp' num2str(i) num2str(j) '=intersect(ids' num2str(i) ', idb' num2str(j) ');']);
       
    end
end

%---------------------------------------------------------------------------
% 
function [p6]   = p6Constr(IndStru, tSpan, P, Ret);

% PORTCONSTR:
%   construct 6 size/bm portfolios using structural cell index across the time range in tSpan
%
% OUTPUT:
%   p6 : 2 by 3 by length(tSpan) portfolio returns
% 
%  Xiaoji lin and Lu Zhang 2011

[N, T]  = size(Ret);
p6 = zeros([2 3 length(tSpan)]);
for i = 1 : 2
    for j = 1 : 3
        % value-weighted returns
         eval(['p6(' num2str(i) ',' num2str(j) ', :) = sum(P(IndStru.idp' num2str(i) num2str(j) ...
            ', tSpan).*Ret(IndStru.idp' num2str(i) num2str(j) ', tSpan)) ./sum(P(IndStru.idp' ...
            num2str(i) num2str(j) ', tSpan));']);
       
        % equal-weighted returns
        % eval(['p6(' num2str(i) ',' num2str(j) ', :) = mean(Ret(IndStru.idp' num2str(i) num2str(j) ', tSpan));']);
    end
end





















